#pragma once

#include "Log.h"
#include <Adafruit_NeoPixel.h>
#include <time.h>
#include <WiFi.h>    // Needed for any NTP usage

class TimeVisualizer {

private:
  static const int LED_PIN_TREBLE = 15;  // or whatever pin you’re using
  static const int LED_COUNT = 12;
  static const int LED_BRIGHTNESS = 150;

  static constexpr char* ntpServer = "pool.ntp.org";
  static const long  gmtOffset_sec = 2 * 3600; // adjust for your timezone
  static const int   daylightOffset_sec = 3600;

  Adafruit_NeoPixel _lightStrip;

  bool _initialized;
  long _lastSetTime;

  // Example color array (red, orange, yellow, green, blue)
  inline static constexpr int colors[5] = {
    0xFF0000,
    0xFFA500,
    0xFFFF00,
    0x00FF00,
    0x0000FF
  };

public:
  TimeVisualizer()
    : _lightStrip(LED_COUNT, LED_PIN_TREBLE, NEO_GRB + NEO_KHZ800),
      _initialized(false),
      _lastSetTime(0) {
    LOG_INFO("TimeVisualizer - Constructor", "Constructing TimeVisualizer...");
  }

  ~TimeVisualizer() {
    LOG_INFO("TimeVisualizer - Destructor", "Destroying TimeVisualizer...");
    resetLedStrip();
  }

  bool setup(const char* ssid, const char* password) {
    LOG_INFO("TimeVisualizer - setup", "starting initialization");
    if (!setupLocalTime(ssid, password)) {
      LOG_ERROR("TimeVisualizer - setup", "failed to setup local time");
      return false;
    }
    if (!setupLightStrip()) {
      LOG_ERROR("TimeVisualizer - setup", "failed to setup LED strip");
      return false;
    }
    _initialized = true;
    LOG_INFO("TimeVisualizer - setup", "Initialization complete!");
    return true;
  }

  // If you only want to show a "time difference" and skip NTP,
  // you could do a custom init that doesn't call setupLocalTime(),
  // and just calls setupLightStrip(). For example:
  /*
  bool setupLightStripOnly() {
    return setupLightStrip();
  }
  */

  bool loop() {
    if (!_initialized) {
      LOG_ERROR("TimeVisualizer - loop", "Not initialized. Call setup() first.");
      return false;
    }
    // Example usage: update LED strip with current time once a second
    const long currentMillis = millis();
    if (currentMillis - _lastSetTime > 1000) {
      _lastSetTime = currentMillis;

      struct tm timeinfo;
      if (!getLocalTime(&timeinfo)) {
        LOG_ERROR("TimeVisualizer - loop", "Failed to grab localTime");
        return false;
      }
      Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");

      int hours   = timeinfo.tm_hour;
      int minutes = timeinfo.tm_min;
      int seconds = timeinfo.tm_sec;

      setHourAndMinutePins(hours, minutes);
      setSecondsPin(seconds);
      _lightStrip.show();
    }
    return true;
  }

  // -------------------------------------------
  //  New method to show "time difference" in binary
  // -------------------------------------------
  void setTimeDifference(int timeLeft) {
    // If you want to allow usage without NTP or "setup()", remove this check:
    if (!_initialized) {
      LOG_INFO("TimeVisualizer - setTimeDifference",
               "Not fully initialized. If ignoring NTP, ensure LED strip is at least begun!");
      // You could do a partial init if needed, or just ignore.
      // return; 
    }

    resetLedStrip(); // Clear the strip before setting bits

    // Bound the value if you like (0..4095 for 12 LEDs)
    if (timeLeft < 0) {
      timeLeft = 0;
    }
    for (int i = 0; i < LED_COUNT; i++) {
      int bit = (timeLeft >> i) & 1;
      if (bit == 1) {
        // White for '1'
        _lightStrip.setPixelColor(i, 0xFFFFFF);
      } else {
        _lightStrip.setPixelColor(i, 0x000000);
      }
    }
    _lightStrip.show();
  }

private:
  bool setupLocalTime(const char* ssid, const char* password) {
    LOG_INFO("TimeVisualizer - setupLocalTime", "Connecting to WiFi for NTP time");
    WiFi.begin(ssid, password);
    int tryCount = 0;
    while (WiFi.status() != WL_CONNECTED && tryCount < 15) {
      delay(1000);
      Serial.print(".");
      tryCount++;
    }
    Serial.println();

    if (WiFi.status() != WL_CONNECTED) {
      LOG_ERROR("TimeVisualizer - setupLocalTime", "Could not connect to WiFi");
      return false;
    }

    configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);

    struct tm timeinfo;
    if(!getLocalTime(&timeinfo)) {
      LOG_ERROR("TimeVisualizer - setupLocalTime", "Failed to obtain time from NTP");
      return false;
    }

    WiFi.disconnect(true);
    WiFi.mode(WIFI_OFF);

    LOG_INFO("TimeVisualizer - setupLocalTime", "Got local time, WiFi disconnected");
    return true;
  }

  bool setupLightStrip() {
    LOG_INFO("TimeVisualizer - setupLightStrip", "Setting up the LED strip");
    _lightStrip.begin();
    _lightStrip.setBrightness(LED_BRIGHTNESS);
    resetLedStrip();
    return true;
  }

  void resetLedStrip() {
    for (int i = 0; i < LED_COUNT; i++) {
      _lightStrip.setPixelColor(i, 0);
    }
    _lightStrip.show();
  }

  void setHourAndMinutePins(int hours, int minutes) {
    // Example from original code:
    int totalTime = (hours + 1) * 100 + minutes;
    for (int i = 0; i < LED_COUNT; ++i) {
      _lightStrip.setPixelColor(i, (totalTime & 1) ? 0xFFFFFF : 0);
      totalTime >>= 1; // shift right by 1
    }
  }

  void setSecondsPin(int seconds) {
    int secondsLedIndex = seconds % LED_COUNT;
    int colorIndex = seconds / LED_COUNT; // picks a color from array
    if (colorIndex >= 5) {
      colorIndex = 4; // clamp to last color if seconds > 59
    }
    _lightStrip.setPixelColor(secondsLedIndex, colors[colorIndex]);
  }
};
